# madcubapy/madcubaio/__init__.py

"""
The `madcubapy.io` package provides classes to read MADCUBA products in
python.
"""

from .madcubafits import *
from .madcubamap import *
from .spectracontainer import *